/* NetworkManager Wireless Applet -- Display wireless access points and allow user control
 *
 * Dan Williams <dcbw@redhat.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * (C) Copyright 2004 Red Hat, Inc.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <libgnomeui/libgnomeui.h>
#include <gnome-keyring.h>

#include <nm-setting-vpn.h>
#include <nm-setting-vpn-properties.h>

#include "../src/nm-vpnc-service.h"
#include "gnome-two-password-dialog.h"

#define KEYRING_CID_TAG "connection-id"
#define KEYRING_SN_TAG "setting-name"
#define KEYRING_SK_TAG "setting-key"

typedef struct {
    const char *password;
    const char *group_password;
    gboolean is_session;
} KeyringData;

void keyring_data_init(KeyringData* d)
{
	memset(d, 0, sizeof(KeyringData));
}

void keyring_data_free(KeyringData* d)
{
	/* For security reasons, Make sure to clear the data before freeing */
	if (d->password) {
		memset((void*)d->password, 0, strlen(d->password));
		g_free((void*)d->password);
		d->password = NULL;
	}
	if (d->group_password) {
		memset((void*)d->group_password, 0, strlen(d->group_password));
		g_free((void*)d->group_password);
		d->group_password = NULL;
	}
}

static char *
find_one_password (const char *vpn_id,
                   const char *vpn_name,
                   const char *vpn_service,
                   const char *secret_name,
                   gboolean *is_session)
{
	GList *found_list = NULL;
	GnomeKeyringResult ret;
	GnomeKeyringFound *found;
	char *secret;

	ret = gnome_keyring_find_itemsv_sync (GNOME_KEYRING_ITEM_GENERIC_SECRET,
	                                      &found_list,
	                                      KEYRING_CID_TAG,
	                                      GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
	                                      vpn_id,
	                                      KEYRING_SN_TAG,
	                                      GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
	                                      NM_SETTING_VPN_PROPERTIES_SETTING_NAME,
	                                      KEYRING_SK_TAG,
	                                      GNOME_KEYRING_ATTRIBUTE_TYPE_STRING,
	                                      secret_name,
	                                      NULL);
	if ((ret != GNOME_KEYRING_RESULT_OK) || (g_list_length (found_list) == 0))
		return NULL;

	found = (GnomeKeyringFound *) found_list->data;

	if (strcmp (found->keyring, "session") == 0)
		*is_session = TRUE;
	else
		*is_session = FALSE;

	secret = found->secret ? g_strdup (found->secret) : NULL;
	gnome_keyring_found_list_free (found_list);

	return secret;
}

void
lookup_pass (const char *vpn_id, const char *vpn_name, const char *vpn_service, KeyringData *kdata)
{
	keyring_data_init(kdata);

	kdata->password = find_one_password (vpn_id, vpn_name, vpn_service, "password", &(kdata->is_session));
	kdata->group_password = find_one_password (vpn_id, vpn_name, vpn_service, "group-password",
						   &(kdata->is_session));
}

static void
save_vpn_password (const char *vpn_id,
                   const char *vpn_name,
                   const char *vpn_service,
                   const char *keyring,
                   const char *secret_name,
                   const char *secret)
{
	char *display_name;
	GnomeKeyringResult ret;
	GnomeKeyringAttributeList *attrs = NULL;
	guint32 id = 0;

	display_name = g_strdup_printf ("VPN %s secret for %s/%s/" NM_SETTING_VPN_PROPERTIES_SETTING_NAME,
	                                secret_name,
	                                vpn_name,
	                                vpn_service);

	attrs = gnome_keyring_attribute_list_new ();
	gnome_keyring_attribute_list_append_string (attrs,
	                                            KEYRING_CID_TAG,
	                                            vpn_id);
	gnome_keyring_attribute_list_append_string (attrs,
	                                            KEYRING_SN_TAG,
	                                            NM_SETTING_VPN_PROPERTIES_SETTING_NAME);
	gnome_keyring_attribute_list_append_string (attrs,
	                                            KEYRING_SK_TAG,
	                                            secret_name);

	ret = gnome_keyring_item_create_sync (keyring,
	                                      GNOME_KEYRING_ITEM_GENERIC_SECRET,
	                                      display_name,
	                                      attrs,
	                                      secret,
	                                      TRUE,
	                                      &id);
	gnome_keyring_attribute_list_free (attrs);
	g_free (display_name);
}

gboolean
get_passwords (const char *vpn_id,
               const char *vpn_name,
               const char *vpn_service,
               gboolean retry,
               KeyringData *kdata)
{
	GSList          *result;
	char            *prompt;
	GtkWidget	*dialog;
	GnomeTwoPasswordDialogRemember remember;

	result = NULL;

	g_return_val_if_fail (vpn_id != NULL, FALSE);
	g_return_val_if_fail (vpn_name != NULL, FALSE);

	lookup_pass (vpn_id, vpn_name, vpn_service, kdata);

	/* If this is not a retry and we have a keyring password,
	   don't prompt and attempt to connect */
	if (!retry && kdata->password)
		return TRUE;

	prompt = g_strdup_printf (_("You need to authenticate to access the Virtual Private Network '%s'."), vpn_name);
	dialog = gnome_two_password_dialog_new (_("Authenticate VPN"), prompt, NULL, NULL, FALSE);
	g_free (prompt);

	gnome_two_password_dialog_set_show_username (GNOME_TWO_PASSWORD_DIALOG (dialog), FALSE);
	gnome_two_password_dialog_set_show_userpass_buttons (GNOME_TWO_PASSWORD_DIALOG (dialog), FALSE);
	gnome_two_password_dialog_set_show_domain (GNOME_TWO_PASSWORD_DIALOG (dialog), FALSE);
	gnome_two_password_dialog_set_show_remember (GNOME_TWO_PASSWORD_DIALOG (dialog), TRUE);
	gnome_two_password_dialog_set_password_secondary_label (GNOME_TWO_PASSWORD_DIALOG (dialog), _("_Group Password:"));
	/* use the same keyring storage options as from the items we put in the entry boxes */
	remember = GNOME_TWO_PASSWORD_DIALOG_REMEMBER_NOTHING;
	if (kdata->group_password)
		remember = GNOME_TWO_PASSWORD_DIALOG_REMEMBER_GROUP;
	else if (kdata->is_session)
		remember = GNOME_TWO_PASSWORD_DIALOG_REMEMBER_SESSION;
	else if (kdata->password)
		remember = GNOME_TWO_PASSWORD_DIALOG_REMEMBER_FOREVER;				

	gnome_two_password_dialog_set_remember (GNOME_TWO_PASSWORD_DIALOG (dialog), remember);

	/* if retrying, put in the passwords from the keyring */
	if (kdata->password)
		gnome_two_password_dialog_set_password (GNOME_TWO_PASSWORD_DIALOG (dialog),
							kdata->password);

	if (kdata->group_password)
		gnome_two_password_dialog_set_password_secondary (GNOME_TWO_PASSWORD_DIALOG (dialog),
								  kdata->group_password);

	gtk_widget_show (dialog);

	if (gnome_two_password_dialog_run_and_block (GNOME_TWO_PASSWORD_DIALOG (dialog)))
	{
		char *password;
		char *group_password;

		password = gnome_two_password_dialog_get_password (GNOME_TWO_PASSWORD_DIALOG (dialog));
		group_password = gnome_two_password_dialog_get_password_secondary (GNOME_TWO_PASSWORD_DIALOG (dialog));
		keyring_data_free(kdata);
		kdata->password = g_strdup(password);
		kdata->group_password = g_strdup(group_password);


		switch (gnome_two_password_dialog_get_remember (GNOME_TWO_PASSWORD_DIALOG (dialog)))
		{
			case GNOME_TWO_PASSWORD_DIALOG_REMEMBER_GROUP:
				save_vpn_password (vpn_id, vpn_name, vpn_service, NULL, "group-password", group_password);
				break;
			case GNOME_TWO_PASSWORD_DIALOG_REMEMBER_SESSION:
				save_vpn_password (vpn_id, vpn_name, vpn_service, "session", "password", password);
				save_vpn_password (vpn_id, vpn_name, vpn_service, "session", "group-password", group_password);
				break;
			case GNOME_TWO_PASSWORD_DIALOG_REMEMBER_FOREVER:
				save_vpn_password (vpn_id, vpn_name, vpn_service, NULL, "password", password);
				save_vpn_password (vpn_id, vpn_name, vpn_service, NULL, "group-password", group_password);
				break;
			default:
				break;
		}

	}

	gtk_widget_destroy (dialog);

	return TRUE;
}

int 
main (int argc, char *argv[])
{
	static gboolean retry = FALSE;
	static gchar *vpn_name = NULL;
	static gchar *vpn_id = NULL;
	static gchar *vpn_service = NULL;
	GOptionContext *context;
	GnomeProgram *program;
	GOptionEntry entries[] =
		{
			{ "reprompt", 'r', 0, G_OPTION_ARG_NONE, &retry, "Reprompt for passwords", NULL},
			{ "id", 'i', 0, G_OPTION_ARG_STRING, &vpn_id, "ID of VPN connection", NULL},
			{ "name", 'n', 0, G_OPTION_ARG_STRING, &vpn_name, "Name of VPN connection", NULL},
			{ "service", 's', 0, G_OPTION_ARG_STRING, &vpn_service, "VPN service type", NULL},
			{ NULL }
		};
	char buf[1];
	KeyringData keyring_data;
	gboolean success = FALSE;

	bindtextdomain (GETTEXT_PACKAGE, NULL);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	context = g_option_context_new ("- vpnc auth dialog");
	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
	
	program = gnome_program_init ("nm-vpnc-auth-dialog", VERSION,
				      LIBGNOMEUI_MODULE,
				      argc, argv,
				      GNOME_PARAM_GOPTION_CONTEXT, context,
				      GNOME_PARAM_NONE);
	  
	if (vpn_id == NULL || vpn_name == NULL || vpn_service == NULL) {
		fprintf (stderr, "Have to supply ID, name, and service\n");
		goto out;
	}

	if (strcmp (vpn_service, NM_DBUS_SERVICE_VPNC) != 0) {
		fprintf (stderr, "This dialog only works with the '%s' service\n", NM_DBUS_SERVICE_VPNC);
		goto out;		
	}

	keyring_data_init(&keyring_data);

	if (!get_passwords (vpn_id, vpn_name, vpn_service, retry, &keyring_data))
		goto out;

	success = TRUE;
	/* dump the passwords to stdout */

	printf ("%s\n%s\n", NM_VPNC_KEY_SECRET, keyring_data.group_password);
	printf ("%s\n%s\n", NM_VPNC_KEY_XAUTH_PASSWORD, keyring_data.password);
	printf ("\n\n");

	/* for good measure, flush stdout since Kansas is going Bye-Bye */
	fflush (stdout);

	keyring_data_free(&keyring_data);

	/* wait for data on stdin  */
	fread (buf, sizeof (char), sizeof (buf), stdin);

out:
	g_object_unref (program);

	return (success ? 0 : 1);
}
